import time
from pinpong.board import Board, UART
Board().begin()  # 初始化，选择板型，不输入板型则进行自动识别
# 硬串口1 P0-RX P3-TX
uart1 = UART(0)
uart1.init()  # 默认波特率为9600


def calculate_checksum(data):
    checksum = 0xFFFF  # 初始校验码值

    for byte in data:
        checksum ^= byte  # 异或操作
        for _ in range(8):
            if checksum & 0x0001:  # 最低位为1
                checksum >>= 1  # 右移一位
                checksum ^= 0xA001  # 异或操作
            else:
                checksum >>= 1  # 右移一位

    return checksum.to_bytes(2, byteorder='little')  # 返回校验码的字节表示，小端序

def validate_response_frame(frame):
    if len(frame) < 5:
        return False
    received_checksum = frame[-2:]  # 从应答帧中提取接收到的校验码
    expected_checksum = calculate_checksum(frame[:-2])  # 计算期望的校验码
    return received_checksum == expected_checksum


while True:
    query_frame = [0x02, 0x03, 0x00, 0x00, 0x00, 0x01, 0x84, 0x39]   # 查询帧
    uart1.write(query_frame)  # 发送查询帧
    time.sleep(0.1)
    try:
        response_frame = uart1.read(7)  # 读取从机应答帧

        if validate_response_frame(bytes(response_frame)): # 与USB通讯不一样的，需要转化成字节串
            byte4 = response_frame[3]  # 获取第4位数据
            byte5 = response_frame[4]  # 获取第5位数据
            

            value = (( byte4<< 8) + byte5) / 10
            print("风速值：", value)
        else:
            print("应答帧校验失败")
        time.sleep(0.1)
    except:
        print("无数据")
