import serial
import time

ser = serial.Serial('/dev/ttyUSB0', 9600, timeout=0.5)

def calculate_checksum(data):
    checksum = 0xFFFF  # 初始校验码值

    for byte in data:
        checksum ^= byte  # 异或操作
        for _ in range(8):
            if checksum & 0x0001:  # 最低位为1
                checksum >>= 1  # 右移一位
                checksum ^= 0xA001  # 异或操作
            else:
                checksum >>= 1  # 右移一位

    return checksum.to_bytes(2, byteorder='little')  # 返回校验码的字节表示，小端序

def validate_response_frame(frame):
    if len(frame) < 5:
        return False
    received_checksum = frame[-2:]  # 从应答帧中提取接收到的校验码
    expected_checksum = calculate_checksum(frame[:-2])  # 计算期望的校验码
    return received_checksum == expected_checksum
while True:
    query_frame = [0x02, 0x03, 0x00, 0x00, 0x00, 0x01, 0x84, 0x39]   # 查询帧

    ser.write(query_frame)  # 发送查询帧
    time.sleep(0.1)
    try:
        response_frame = ser.read(7)  # 读取从机应答帧
        if validate_response_frame(response_frame):
            byte4 = response_frame[3]  # 获取第4位数据
            byte5 = response_frame[4]  # 获取第5位数据

            value = (( byte4<< 8) + byte5) / 10
            print("风速值：", value)
        else:
            print("应答帧校验失败")
        time.sleep(0.1)
    except:
        print("无数据")

